************************************************************************
      PROGRAM lomb
* Lomb-Scargle + pikaia
* Finding periods in a signal
* Output files:
* *.fre: list of lines found:
*        freq, period, amp, estim err amp, phase, estim err phase
* *.out: verbose output of the run (some more info than to screen)
* *.pgm: periodograms:
*        freq, original pgm, final pgm, pgm of the observing window
* *.rsd: residuals:
*        t, obs, synthetic, obs - synt
* *.sgn: synthetic signal:
*        t, Sum [a * cos(omega t + phi)]
************************************************************************
      INCLUDE 'signum.par'
* MPI
      INTEGER nodo,numprocs
      COMMON /quiensoy/nodo,numprocs
      INCLUDE 'mpif.h'
D     INTEGER mcw,mdp
D     PARAMETER (mcw=MPI_COMM_WORLD,mdp=MPI_DOUBLE_PRECISION)
D     INTEGER ierr
* Shared with pikaia
      INTEGER nt,nf,jfmax
      DOUBLE PRECISION obs(nmax),tt(nmax),pes(nmax),limin(3),limax(3),
     & df,sig(nmax)
      COMMON /datos/obs,tt,pes,df,limin,limax,nt,nf,jfmax

      LOGICAL har
      CHARACTER arch*30
      INTEGER i,ind(nmax),estatus,hay,k,j,m,nlin,ju,ipan
      DOUBLE PRECISION fmax,dtmax,nyquist,pg(0:nfmax),pmax,pot0,
     & aux(nmax),ctrl(14),af,amfa(3),fit,ugen(3,npp),fitug(npp),
     & ww,dpi,fre(maxlin),amp(maxlin),fas(maxlin),obs0(nmax),prob,
     & synt,ttt,cont(maxlin),pg0(0:nfmax),resf,potf,fcom,tol,sa,sf,
     & pw(0:nfmax),freli
      EXTERNAL af

* Start MPI.
      nodo=0
      numprocs=1
D     CALL MPI_INIT(ierr)
* Process number of this process.
D     CALL MPI_COMM_RANK(mcw,nodo,ierr)
* Number of processors.
D     CALL MPI_COMM_SIZE(mcw,numprocs,ierr)

      dpi=2d0*ACOS(-1d0)
* Pikaia's parameters 
      ctrl(1)=0d0 ! random or read
      ctrl(2)=1000d0 ! generations
      ctrl(3)=9d0 ! digits
      ctrl(4)=1d0 ! crossover
      ctrl(5)=5d0 ! mutation mode
      ctrl(7)=0.0005d0 ! minimum mutation
      ctrl(8)=0.03d0 ! maximum mutation
      ctrl(9)=0.85d0 ! fitness differential
      ctrl(10)=4d0 ! reproduction plan
      ctrl(11)=1d0 ! elitism
      ctrl(12)=0d0 ! selection method
      ctrl(13)=0d0 ! output
      ctrl(14)=0d0 ! MPI
* Input data
      OPEN(50,FILE='signum.in')
      READ(50,*)arch
      READ(50,*)hay
      READ(50,*)ipan
      CLOSE(50)
      ju=INDEX(arch,' ')-1
      OPEN(50,FILE=arch,STATUS='OLD')
      ww=0d0
      i=1
      DOWHILE(.TRUE.)
         IF(hay.EQ.1)THEN
            READ(50,*,END=1)tt(i),obs(i),sig(i)
         ELSE
            READ(50,*,END=1)tt(i),obs(i)
            sig(i)=1d0
         ENDIF
         ww=ww+1d0/sig(i)**2
         IF(i.GT.nmax)THEN
            IF(nodo.EQ.0)PRINT *,'Increase nmax'
D           CALL MPI_FINALIZE(ierr)
            STOP
         ENDIF
         i=i+1
      ENDDO
1     nt=i-1
* Weight normalization
      DO i=1,nt
         pes(i)=1d0/(ww*sig(i)**2)
      ENDDO
      CLOSE(50)
* Sort
      DO i=1,nt
         aux(i)=obs(i)
      ENDDO
      CALL quicki8(nt,tt,ind)
      DO i=1,nt
         obs(i)=aux(ind(i))
         obs0(i)=obs(i)
      ENDDO
* Grid in frequency
      fmax=nyquist(nt,tt) ! maximum freq
      dtmax=tt(nt)-tt(1)  ! total time span
      df=1d0/(ovs*dtmax)  ! minimum (delta) freq
      nf=INT(MIN(DBLE(nfmax),fmax/df))     ! number of freq to evaluate

      IF(nodo.EQ.0)THEN
         OPEN(60,FILE=arch(1:ju)//'.out')
         WRITE(60,*)'Search between frequencies ',df,' and',nf*df
         WRITE(60,*)nf,' frequencies'
         WRITE(60,*)'First search (raw)'
         WRITE(60,*)' Freq                       Period',
     &    '                    Amplitude                 Phase',
     &    '                    Continuum                 Probability'
         IF(ipan.EQ.1)THEN
            PRINT *,'Search between frequencies ',REAL(df),' and ',
     &       REAL(nf*df)
            PRINT *,'First search (raw)'
            PRINT *,' Freq             Period          Amplitude',
     &       '      Probability'
D           CALL flush(6)
         ENDIF
      ENDIF

      nlin=0
      DO k=1,maxlin
* Periodogram
         CALL pgramwc(nt,tt,obs,pes,df,nf,pg(1),pmax,jfmax,prob)
* Store the original periodogram
         IF(k.EQ.1)THEN
            DO i=0,nf
               pg0(i)=pg(i)
            ENDDO
         ENDIF
* Low probability
         IF(prob.LT.plim)THEN
            IF(nodo.EQ.0)WRITE(60,*)'Next line probability = ',prob
            nlin=k-1
            GOTO 2
         ENDIF
* Search for the exact frequency
         limin(1)=df*(jfmax-1)
         limax(1)=df*(jfmax+1)
         ctrl(6)=0.005d0 ! initial mutation
         CALL pikaia8(af,1,ctrl,amfa,fit,estatus,npp,ugen,fitug)
         fre(k)=amfa(1)*(limax(1)-limin(1))+limin(1)
* Amplitude and phase
         CALL ampfas(nt,tt,obs,pes,ww,fre(k),amp(k),fas(k),cont(k),sa,
     &    sf)
         fas(k)=MOD(fas(k)+dpi,dpi)

         IF(nodo.EQ.0)THEN
            WRITE(60,*)fre(k),1d0/fre(k),amp(k),fas(k),cont(k),prob
            IF(ipan.EQ.1)THEN
               PRINT *,REAL(fre(k)),REAL(1d0/fre(k)),REAL(amp(k)),
     &          REAL(prob)
D              CALL flush(6)
            ENDIF
         ENDIF

* Verify whether it is the debris of an already extracted line
         DO j=1,k-1
            IF(ABS(fre(k)-fre(j)).LT.5*df)THEN
               nlin=k-1
               GOTO 2
            ENDIF
         ENDDO
* Substract line
         DO i=1,nt
            obs(i)=obs(i)-amp(k)*COS(dpi*fre(k)*tt(i)+fas(k))-cont(k)
         ENDDO
      ENDDO
2     CONTINUE

      IF(nodo.EQ.0)THEN
         WRITE(60,*)'# lines = ',nlin
         IF(ipan.EQ.1)THEN
            PRINT *,'# lines = ',nlin
D           CALL flush(6)
         ENDIF
      ENDIF

* Refinement
      OPEN(61,FILE=arch(1:ju)//'.fre')
      DO m=1,mmax

         IF(nodo.EQ.0)THEN
            WRITE(60,*)'Refinement, pass ',m
            IF(ipan.EQ.1)PRINT *,'Refinement, pass ',m
            IF(m.EQ.mmax)THEN
               WRITE(60,*)'Last search'
               WRITE(60,*)' Freq                      Period',
     & '                    Amplitude                 err   ',
     & '                    Phase                     err   ',
     & '                    Continuum'
               IF(ipan.EQ.1)THEN
                  PRINT *,'Last search'
                  PRINT *,' Freq             Period          Amplitude',
     &             '      Continuum'
               ENDIF
            ENDIF
D           CALL flush(6)
         ENDIF

         DO k=1,nlin
            DO i=1,nt
               obs(i)=obs0(i)
            ENDDO
* Substract all lines except one
            DO j=1,nlin
               IF(j.NE.k)THEN
                  DO i=1,nt
                     obs(i)=obs(i)-amp(j)*COS(dpi*fre(j)*tt(i)+fas(j))-
     &                cont(j)
                  ENDDO
               ENDIF
            ENDDO
* Periodogram
            CALL pgramwc(nt,tt,obs,pes,df,nf,pg(1),pmax,jfmax,prob)
* Search for the exact frequency
            limin(1)=df*(jfmax-1)
            limax(1)=df*(jfmax+1)
            ctrl(6)=0.005d0 ! initial mutation
            CALL pikaia8(af,1,ctrl,amfa,fit,estatus,npp,ugen,fitug)
            fre(k)=amfa(1)*(limax(1)-limin(1))+limin(1)
* Amplitude and phase
            CALL ampfas(nt,tt,obs,pes,ww,fre(k),amp(k),fas(k),cont(k),
     &       sa,sf)
            fas(k)=MOD(fas(k)+dpi,dpi)

            IF(m.EQ.mmax.AND.nodo.EQ.0)THEN
               WRITE(60,*)fre(k),1d0/fre(k),amp(k),sa,fas(k),sf,cont(k)
               WRITE(61,*)fre(k),1d0/fre(k),amp(k),sa,fas(k),sf
               IF(ipan.EQ.1)THEN
                  PRINT *,REAL(fre(k)),REAL(1d0/fre(k)),REAL(amp(k)),
     &             REAL(cont(k))
D                 CALL flush(6)
               ENDIF
            ENDIF

         ENDDO
      ENDDO
      CLOSE(61)

* Possible window interactions
      IF(nodo.EQ.0)THEN
         WRITE(60,*)'Possible aliases due to the observing window:'
         IF(ipan.EQ.1)THEN
            PRINT *,'Possible aliases due to the observing window:'
D           CALL flush(6)
         ENDIF
      ENDIF
* Window transform
      CALL pgram(nt,tt,df,nf,pw(1),pmax,jfmax)
      tol=df
      freli=df*jfmax
      har=.FALSE.
      DO m=1,nlin
         IF(ABS(fre(m)-freli).LT.tol)THEN
            har=.TRUE.
            IF(nodo.EQ.0)THEN
               WRITE(60,*)fre(m),' = w',freli
               IF(ipan.EQ.1)THEN
                  PRINT *,REAL(fre(m)),' = w',REAL(freli)
D                 CALL flush(6)
               ENDIF
            ENDIF
         ENDIF
      ENDDO
      DO k=1,nlin
         DO i=-3,3
            IF(i.NE.0)THEN
               DO j=1,2
                  fcom=ABS(j*fre(k)+i*freli)
                  DO m=1,nlin
                     IF(m.NE.k)THEN
                        IF(ABS(fre(m)-fcom).LT.tol)THEN
                           har=.TRUE.
                           IF(nodo.EQ.0)THEN
                              WRITE(60,*)fre(m),' = ',fre(k),' + ',i,
     &                      ' x w',freli
                              IF(ipan.EQ.1)THEN
                                PRINT *,REAL(fre(m)),' = ',REAL(fre(k)),
     &                           ' + ',i,' x w',REAL(freli)
D                                CALL flush(6)
                              ENDIF
                           ENDIF
                        ENDIF
                     ENDIF
                  ENDDO
               ENDDO
            ENDIF
         ENDDO
      ENDDO
      IF(.NOT.har)THEN
         IF(nodo.EQ.0)THEN
            WRITE(60,*)'none.'
            IF(ipan.EQ.1)THEN
               PRINT *,'none.'
D              CALL flush(6)
            ENDIF
         ENDIF
      ENDIF

* Final outputs
      IF(nodo.EQ.0)THEN
* Original and final periodograms
         DO i=1,nt
            obs(i)=obs0(i)
         ENDDO
         DO j=1,nlin
            DO i=1,nt
               obs(i)=obs(i)-amp(j)*COS(dpi*fre(j)*tt(i)+fas(j))-cont(j)
            ENDDO
         ENDDO
         CALL pgramwc(nt,tt,obs,pes,df,nf,pg(1),pmax,jfmax,prob)
         OPEN(61,FILE=arch(1:ju)//'.pgm')
         potf=0d0 ! total power
         pot0=0d0 ! total power
         DO i=0,nf
            WRITE(61,*)df*i,pg0(i),pg(i),pw(i)
            potf=potf+pg(i)
            pot0=pot0+pg0(i)
         ENDDO
         CLOSE(61)
* Output of the residuals
         OPEN(61,FILE=arch(1:ju)//'.rsd')
         resf=0d0 ! total residuals
         DO i=1,nt
            synt=0d0
            DO j=1,nlin
               synt=synt+amp(j)*COS(dpi*fre(j)*tt(i)+fas(j))+cont(j)
            ENDDO
            WRITE(61,*)tt(i),obs0(i),synt,obs0(i)-synt
            resf=resf+ABS(obs0(i)-synt)
         ENDDO
         CLOSE(61)
* Output of the signal found
         OPEN(61,FILE=arch(1:ju)//'.sgn')
         DO i=0,nf
            ttt=i*(tt(nt)-tt(1))/DBLE(nf)+tt(1)
            synt=0d0
            DO j=1,nlin
               synt=synt+amp(j)*COS(dpi*fre(j)*ttt+fas(j))+cont(j)
            ENDDO
            WRITE(61,*)ttt,synt
         ENDDO
         CLOSE(61)
         WRITE(60,*)'Estimated frequency error = ',df*ovs
         WRITE(60,*)'Total residuals = ',resf
         WRITE(60,*)'Orig. power, final power, ratio = ',pot0,potf,
     &    potf/pot0
         WRITE(60,*)'Power of noise if Gaussian = ',nf
         IF(ipan.EQ.1)THEN
            PRINT *,'Estimated frequency error = ',df*ovs
            PRINT *,'Total residuals = ',REAL(resf)
            PRINT *,'Pow_0, Pow_f, ratio = ',REAL(pot0),REAL(potf),
     &       REAL(potf/pot0)
D           CALL flush(6)
         ENDIF
      ENDIF
      CLOSE(61)
* Close MPI
D     CALL MPI_FINALIZE(ierr)
      END

************************************************************************
      FUNCTION nyquist(nt,tt)
      INTEGER nt
      DOUBLE PRECISION nyquist,tt(nt)
* It computes the maximum frequency for a non-uniform distribution of
* observation times
************************************************************************
      DOUBLE PRECISION df
      PARAMETER (df=1d-6)

      INTEGER gcd,i,kdel,euclides

      gcd=NINT((tt(2)-tt(1))/df)
      DO i=3,nt
         kdel=NINT((tt(i)-tt(i-1))/df)
         gcd=euclides(kdel,gcd)
      ENDDO
      nyquist=0.5d0/(gcd*df)
      END
************************************************************************
      FUNCTION euclides(u0,v0)
      INTEGER euclides,u0,v0
* Find the g.c.d. between u0 and v0, using Euclids' algorithm
************************************************************************
      INTEGER aux,u,v
      u=u0
      v=v0
      DOWHILE(v.NE.0)
         aux=MOD(u,v)
         u=v
         v=aux
      ENDDO
      euclides=u
      END

************************************************************************
      SUBROUTINE pgramwc(nt,tt,obs,pes,df,nf,pg,pmax,jmax,prob)
      INTEGER nt,nf,jmax
      DOUBLE PRECISION tt(nt),obs(nt),df,pg(nf),pmax,pes(nt),prob
* Periodogram with weights (errors) and floating mean
* Input:
* nt number of observations
* tt(nt) times
* obs(nt) observations
* pes(nt) errors
* df delta frequency
* nf number of frequencies
* Output:
* pg(nf) set of powers
* pmax maximum power
* jmax position of pmax
* prob probability of not being noise
************************************************************************
      INCLUDE 'signum.par'

      LOGICAL prim
      INTEGER i,j
      DOUBLE PRECISION dpi,arg,wpr(nmax),wpi(nmax),wr(nmax),
     & wi(nmax),sums,sumc,c,s,wtau,swtau,cwtau,sumys,st,ct,
     & sumyc,ss,cc,y,wtemp,sumss,sumcc,sumcs,cs,ys,yc,p,expy,effm,
     & sumy,sumyy,yy
      SAVE prim,dpi
      DATA prim/.TRUE./

      IF(prim)THEN
         prim=.FALSE.
         dpi=2d0*ACOS(-1d0)
      ENDIF

* Initialization
      DO i=1,nt
         arg=dpi*df*tt(i)
         wpr(i)=-2d0*SIN(0.5d0*arg)**2
         wpi(i)=SIN(arg)
         wr(i)=COS(arg)
         wi(i)=wpi(i)
      ENDDO
* Loop over frequencies
      pmax=0d0 ! maximum of the periodogram
      DO i=1,nf
* Tau
         sums=0d0
         sumc=0d0
         sumss=0d0
         sumcc=0d0
         sumcs=0d0
         DO j=1,nt
            c=wr(j)
            s=wi(j)
            p=pes(j)
            sums=sums+p*s
            sumc=sumc+p*c
            sumss=sumss+p*s**2
            sumcs=sumcs+p*s*c
            sumcc=sumcc+p*c**2
         ENDDO
         ss=sumss-sums**2
         cc=sumcc-sumc**2
         cs=sumcs-sums*sumc
         wtau=0.5d0*ATAN2(2d0*cs,cc-ss)
         swtau=SIN(wtau)
         cwtau=COS(wtau)
* Periodogram
         sums=0d0
         sumc=0d0
         sumy=0d0
         sumss=0d0
         sumcc=0d0
         sumys=0d0
         sumyc=0d0
         sumyy=0d0
         DO j=1,nt
            s=wi(j)
            c=wr(j)
            y=obs(j)
            p=pes(j)
            st=s*cwtau-c*swtau
            ct=c*cwtau+s*swtau
            sums=sums+p*st
            sumc=sumc+p*ct
            sumy=sumy+p*y
            sumss=sumss+p*st**2
            sumcc=sumcc+p*ct**2
            sumys=sumys+p*y*st
            sumyc=sumyc+p*y*ct
            sumyy=sumyy+p*y**2
            wtemp=wr(j)
            wr(j)=(wr(j)*wpr(j)-wi(j)*wpi(j))+wr(j)
            wi(j)=(wi(j)*wpr(j)+wtemp*wpi(j))+wi(j)
         ENDDO
         ss=sumss-sums**2
         cc=sumcc-sumc**2
         ys=sumys-sumy*sums
         yc=sumyc-sumy*sumc
         yy=sumyy-sumy**2
         pg(i)=0.5d0*(yc**2/cc+ys**2/ss)/(yy/(nt-1))
         IF(pg(i).GE.pmax.AND.i.NE.1)THEN
            pmax=pg(i)
            jmax=i
         ENDIF
      ENDDO
      expy=EXP(-pmax)
      effm=2d0*nf/DBLE(ovs)
      prob=effm*expy
      IF(prob.GT.0.01d0)prob=1d0-(1d0-expy)**effm
      prob=1d0-prob
      END

************************************************************************
      FUNCTION af(n,x)
      INTEGER n
      DOUBLE PRECISION x(n),af
* n = 1
* x(1) = frequency
************************************************************************
      INCLUDE 'signum.par'

      INTEGER nt,nf,jfmax
      DOUBLE PRECISION obs(nmax),tt(nmax),pes(nmax),limin(3),limax(3),
     & df
      COMMON /datos/obs,tt,pes,df,limin,limax,nt,nf,jfmax

      LOGICAL prim
      INTEGER i,j
      DOUBLE PRECISION dpi,arg,wr(nmax),fre,sums,sumc,p,
     & wi(nmax),c,s,sumys,wtau,swtau,cwtau,st,ct,sumy,
     & sumyc,ss,cc,y,sumss,sumcc,sumcs,cs,ys,yc
      SAVE prim,dpi
      DATA prim/.TRUE./

      IF(prim)THEN
         dpi=2d0*ACOS(-1d0)       
         prim=.FALSE.
      ENDIF
* Unfold variables
      fre=x(1)*(limax(1)-limin(1))+limin(1)
      DO i=1,nt
         arg=dpi*fre*tt(i)
         wr(i)=COS(arg)
         wi(i)=SIN(arg)
      ENDDO
* Tau
      sums=0d0
      sumc=0d0
      sumss=0d0
      sumcc=0d0
      sumcs=0d0
      DO j=1,nt
         c=wr(j)
         s=wi(j)
         p=pes(j)
         sums=sums+p*s
         sumc=sumc+p*c
         sumss=sumss+p*s**2
         sumcs=sumcs+p*s*c
         sumcc=sumcc+p*c**2
      ENDDO
      ss=sumss-sums**2
      cc=sumcc-sumc**2
      cs=sumcs-sums*sumc
      wtau=0.5d0*ATAN2(2d0*cs,cc-ss)
      swtau=SIN(wtau)
      cwtau=COS(wtau)
* Power
      sums=0d0
      sumc=0d0
      sumy=0d0
      sumss=0d0
      sumcc=0d0
      sumys=0d0
      sumyc=0d0
      DO j=1,nt
         s=wi(j)
         c=wr(j)
         y=obs(j)
         p=pes(j)
         st=s*cwtau-c*swtau
         ct=c*cwtau+s*swtau
         sums=sums+p*st
         sumc=sumc+p*ct
         sumy=sumy+p*y
         sumss=sumss+p*st**2
         sumcc=sumcc+p*ct**2
         sumys=sumys+p*y*st
         sumyc=sumyc+p*y*ct
      ENDDO
      ss=sumss-sums**2
      cc=sumcc-sumc**2
      ys=sumys-sums*sumy
      yc=sumyc-sumc*sumy
      af=yc**2/cc+ys**2/ss
      END

************************************************************************
      SUBROUTINE ampfas(nt,tt,obs,pes,ww,fre,amp,fas,cont,sigamp,sigfas)
      INTEGER nt
      DOUBLE PRECISION fre,tt(nt),obs(nt),pes(nt),ww,amp,fas,cont,
     & sigamp,sigfas
* Input: nt,ff,obs,pes,fre,ww
* Output: amp,fas,cont,sigamp,sigfas
************************************************************************
      INCLUDE 'signum.par'

      LOGICAL prim
      INTEGER i,j
      DOUBLE PRECISION dpi,arg,wr(nmax),wi(nmax),sums,sumc,sumss,sumcc,
     & sumcs,c,s,sumy,sumys,sumyc,y,aa,bb,dd,p,sigaa,sigbb
      SAVE prim,dpi
      DATA prim/.TRUE./

      IF(prim)THEN
         dpi=2d0*ACOS(-1d0)       
         prim=.FALSE.
      ENDIF
      DO i=1,nt
         arg=dpi*fre*tt(i)
         wr(i)=COS(arg)
         wi(i)=SIN(arg)
      ENDDO
* Power
      sums=0d0
      sumc=0d0
      sumy=0d0
      sumss=0d0
      sumcc=0d0
      sumcs=0d0
      sumys=0d0
      sumyc=0d0
      DO j=1,nt
         s=wi(j)
         c=wr(j)
         y=obs(j)
         p=pes(j)
         sums=sums+p*s
         sumc=sumc+p*c
         sumy=sumy+p*y
         sumss=sumss+p*s**2
         sumcc=sumcc+p*c**2
         sumcs=sumcs+p*s*c
         sumys=sumys+p*y*s
         sumyc=sumyc+p*y*c
      ENDDO
      sumss=sumss-sums**2
      sumcc=sumcc-sumc**2
      sumcs=sumcs-sumc*sums
      sumys=sumys-sums*sumy
      sumyc=sumyc-sumc*sumy
      dd=sumcc*sumss-sumcs**2
      aa=(sumyc*sumss-sumys*sumcs)/dd
      bb=(sumys*sumcc-sumyc*sumcs)/dd
      amp=SQRT(aa**2+bb**2)
      fas=ATAN2(-bb,aa)
      cont=sumy-aa*sumc-bb*sums
* Error estimation
      sigbb=SQRT(sumcc/dd/ww)
      sigaa=SQRT(sumss/dd/ww)
      sigamp=(ABS(aa*sigaa)+ABS(bb*sigbb))/amp
      sigfas=(ABS(aa*sigbb)+ABS(bb*sigaa))/amp**2
      END

************************************************************************
      SUBROUTINE pgram(nt,tt,df,nf,pg,pmax,jmax)
      INTEGER nt,nf,jmax
      DOUBLE PRECISION tt(nt),df,pg(nf),pmax
* Periodogram; no floating mean
* The observations correspond to 1's.
* Input:
* tt(nt) times
* df delta frequency
* nf number of frequencies
* Output:
* pg(nf) set of powers
* pmax maximum power
* jmax position of pmax
************************************************************************
      INCLUDE 'signum.par'

      LOGICAL prim
      INTEGER i,j
      DOUBLE PRECISION dpi,arg,wpr(nmax),wpi(nmax),wr(nmax),
     & wi(nmax),sumc,c,s,wtau,swtau,cwtau,sums,sumsy,
     & sumcy,ss,cc,wtemp
      SAVE prim,dpi
      DATA prim/.TRUE./

      IF(prim)THEN
         prim=.FALSE.
         dpi=2d0*ACOS(-1d0)
      ENDIF
* Initialization
      DO i=1,nt
         arg=dpi*df*tt(i)
         wpr(i)=-2d0*SIN(0.5d0*arg)**2
         wpi(i)=SIN(arg)
         wr(i)=COS(arg)
         wi(i)=wpi(i)
      ENDDO
* Loop over frequencies
      pmax=0d0 ! maximum of the periodogram
      DO i=1,nf
         sums=0d0
         sumc=0d0
* Tau
         DO j=1,nt
            c=wr(j)
            s=wi(j)
            sums=sums+s*c
            sumc=sumc+(c-s)*(c+s)
         ENDDO
         wtau=0.5d0*ATAN2(2d0*sums,sumc)
         swtau=SIN(wtau)
         cwtau=COS(wtau)
* Periodogram
         sums=0d0
         sumc=0d0
         sumsy=0d0
         sumcy=0d0
         DO j=1,nt
            s=wi(j)
            c=wr(j)
            ss=s*cwtau-c*swtau
            cc=c*cwtau+s*swtau
            sums=sums+ss**2
            sumc=sumc+cc**2
            sumsy=sumsy+ss ! = + y*ss = + 1*ss
            sumcy=sumcy+cc ! = + y*cc = + 1*cc
            wtemp=wr(j)
            wr(j)=(wr(j)*wpr(j)-wi(j)*wpi(j))+wr(j)
            wi(j)=(wi(j)*wpr(j)+wtemp*wpi(j))+wi(j)
         ENDDO
         pg(i)=0.5d0*(sumcy**2/sumc+sumsy**2/sums)
         IF(pg(i).GE.pmax.AND.i.GT.2*ovs)THEN
            pmax=pg(i)
            jmax=i
         ENDIF
      ENDDO
      END

      INCLUDE 'quicki8.sub'
      INCLUDE 'perpik.sub'
      INCLUDE 'azar8.fun'
